@@ -1,4 +1,3 @@
-extern crate newton_rootfinder;
 use newton_rootfinder as nrf;
 use util::test_cases::broyden1965::*;
 
@@ -25,7 +24,7 @@ fn broyden_case10_fd() {
 
     let mut user_model = nrf::model::UserModelFromFunction::new(problem_size, broyden1965_case10);
 
-    rf.solve(&mut user_model);
+    rf.solve(&mut user_model).unwrap();
 
     let solution = solution_broyden1965_case10();
 
