@@ -1,8 +1,10 @@
 mod advanced_parametrization;
 mod common;
+mod errors;
 mod jacobian;
 mod log;
 mod model;
+#[cfg(feature = "xml_config_file")]
 mod parser;
 mod solver;
 mod user_model;
